from testharness import import_from_parent, test_run

import_from_parent()

import six
from say import Say, say, fmt

def test_basic(param = 'Yo'):
    say = Say()
    
    greeting = "hello"
    assert say("{greeting}, world!") == "{}, world!".format(greeting)
    assert say("'{greeting}' has {len(greeting)} characters") == "'{}' has {} characters".format(greeting, len(greeting))
    assert say("{param}") == "{}".format(param)
    assert say("{greeting:<10}") == "{:<10}".format(greeting)
    assert say("{greeting:>20}") == "{:>20}".format(greeting)
    
    status = "OK"
    assert say("This is {status}", silent=True) == "This is OK"
    
def test_hr_and_title():
    say = Say()
    
    say.title('say testing follows')
    say.hr(vsep=1)

    # http://en.wikipedia.org/wiki/Box-drawing_character
    say.hr(sep=six.u('\u2504'))
    say.hr(sep=six.u('\u2509'))
    say.hr(sep=six.u('\u2550'))
    say.hr(sep=six.u('\u2591'))
    say.hr(sep=six.u('\u25EF '))
    say.hr(sep=six.u('\u25C9 '))
    
def test_localvar():
    m = "MIGHTY"
    assert say(m) == "MIGHTY"
    assert say("{m}") == "MIGHTY"
    globalvar = "tasty"    # override global var
    assert say(globalvar) == "tasty"
    assert say("{globalvar}") == "tasty"
    
def test_globalvar():
    assert say("{globalvar}") == str(globalvar)
    
def test_unicode():
    say = Say()
    
    u = six.u('This\u2014is Unicode!')
    assert say(u) == u
    assert say("Hey! {u}") == six.u('Hey! This\u2014is Unicode!')
    too = "too"
    assert say(six.u("Unicode templates {too}")) == six.u("Unicode templates too")
    
def test_format_string():
    x = 33.123456789
    assert say("{x} is floating point!") == '33.123456789 is floating point!'
    assert say("{x:0.2f} is shorter") == '33.12 is shorter'
    assert say("{x:8.3f} is in the middle") == '  33.123 is in the middle'
    
def test_fmt():
    v = 1212
    
    assert say("++{v}") == fmt("++{v}")
    # should print only once

def test_example_1():
    say = Say()
    
    x = 12
    nums = range(4)
        
    assert say("There are {x} things.") == "There are 12 things."
    assert say("Nums has {len(nums)} items: {nums}") == "Nums has 4 items: [0, 1, 2, 3]"
    assert say("Or quoted: {[str(x) for x in nums]}") == "Or quoted: ['0', '1', '2', '3']"
     
def test_example_2():
    
    errors = [ {'name': 'I/O Error', 'timestamp': 23489273 },
               {'name': 'Compute Error', 'timestamp': 349734289 }]
    
    say.title('Errors')
    for i,e in enumerate(errors, start=1):
        say("{i:3}: {e['name'].upper()}")
        
def test_say_silent():
    say = Say()
    r1 = say("this is output")
    r2 = say("this is output", silent=True)
    assert r1 == r2
  
if __name__ == '__main__':
    globalvar = 99
    test_run()